import { generateSimpleViteManifest, injectApp, mockConfig, mockFetch, mockRedis } from './util.js'
import { expect } from 'chai'
import * as td from 'testdouble'
import RedisMock from 'ioredis-mock'

describe('Responses contain custom headers', function () {
  let fetchConfig
  let config
  let app

  beforeEach(async function () {
    await mockConfig(config = { baseUrls: ['http://ui-server/'] })
    await mockRedis()
    mockFetch(fetchConfig = {
      'http://ui-server': {
        '/manifest.json': generateSimpleViteManifest({
          'example.js': {}
        }),
        '/example.js': () => new Response('this is example', { headers: { 'content-type': 'application/javascript' } }),
        '/meta.json': { name: 'sample-service', version: '1.0' }
      }
    })
    app = await injectApp()
  })

  afterEach(async function () {
    await new RedisMock().flushdb()
    delete process.env.APP_VERSION
    delete process.env.BUILD_TIMESTAMP
    delete process.env.CI_COMMIT_SHA
    td.reset()
  })

  it('has own metadata', async function () {
    process.env.APP_VERSION = '4.2'
    process.env.BUILD_TIMESTAMP = '0123456789'
    process.env.CI_COMMIT_SHA = '0123456789abcdef'

    const response = await app.inject({ url: '/meta' })
    expect(response.statusCode).to.equal(200)
    expect(response.json()).to.deep.contain({
      id: 'ui-middleware',
      name: 'UI Middleware',
      buildDate: '0123456789',
      commitSha: '0123456789abcdef',
      version: '4.2'
    })
  })

  it('has metadata from another ui service if available', async function () {
    const response = await app.inject({ url: '/meta' })
    expect(response.statusCode).to.equal(200)
    expect(response.json()).to.deep.contain({
      name: 'sample-service',
      version: '1.0'
    })
  })

  it('has updated metadata if config is updated', async function () {
    const response = await app.inject({ url: '/meta' })
    expect(response.json()).to.have.length(2)

    config.baseUrls = []
    await import('../src/version.js').then(async ({ updateVersionProcessor }) => {
      // need to process two times to actually trigger the update
      await updateVersionProcessor()
      await updateVersionProcessor()
    })

    const response2 = await app.inject({ url: '/meta' })
    expect(response2.json()).to.have.length(1)
  })

  describe('without service avaible', function () {
    beforeEach(function () {
      delete fetchConfig['http://ui-server']
    })

    it('does not have metadata from ui service when unavailable', async function () {
      await import('../src/cache.js').then(({ clear }) => clear())
      const response = await app.inject({ url: '/meta' })
      expect(response.statusCode).to.equal(200)
      expect(response.json()).to.not.deep.contain({
        name: 'sample-service',
        version: '1.0'
      })
    })
  })

  describe('without redis disabled', function () {
    beforeEach(async function () {
      td.reset()
      await mockConfig({ baseUrls: ['http://ui-server/'] })
      await mockRedis()
      mockFetch(fetchConfig = {
        'http://ui-server': {
          '/manifest.json': generateSimpleViteManifest({
            'example.js': {}
          }),
          '/example.js': () => new Response('this is example', { headers: { 'content-type': 'application/javascript' } }),
          '/meta.json': { name: 'sample-service', version: '1.0' }
        }
      })
      app = await injectApp()
    })

    it('has metadata', async function () {
      const response = await app.inject({ url: '/meta' })
      expect(response.statusCode).to.equal(200)
      expect(response.json()).to.deep.contain({
        name: 'sample-service',
        version: '1.0'
      })
    })
  })
})
